@@ -54,7 +54,35 @@ class AuthenticatedStrategy(Strategy):
         return driver.get_cookie('li_at') is not None
 
     @staticmethod
-    def __load_job_details(driver: webdriver, job_id: str, timeout=2) -> object:
+    def __load_jobs(driver: webdriver, job_tot: int, timeout=5) -> object:
+        """
+        Load more jobs
+        :param driver: webdriver
+        :param job_tot: int
+        :param timeout: int
+        :return: object
+        """
+
+        elapsed = 0
+        sleep_time = 0.05
+
+        try:
+            while elapsed < timeout:
+                jobs_count = driver.execute_script(
+                        'return document.querySelectorAll(arguments[0]).length;', Selectors.jobs)
+
+                if jobs_count > job_tot:
+                    return {'success': True, 'count': jobs_count}
+
+                sleep(sleep_time)
+                elapsed += sleep_time
+        except:
+            pass
+
+        return {'success': False, 'count': -1}
+
+    @staticmethod
+    def __load_job_details(driver: webdriver, job_id: str, timeout=5) -> object:
         """
         Wait for job details to load
         :param driver: webdriver
@@ -62,59 +90,59 @@ class AuthenticatedStrategy(Strategy):
         :param timeout: int
         :return: object
         """
+
         elapsed = 0
         sleep_time = 0.05
 
-        while elapsed < timeout:
-            loaded = driver.execute_script(
-                '''
-                    const detailsPanel = document.querySelector(arguments[1]);
-                    const description = document.querySelector(arguments[2]);
-                    return detailsPanel && detailsPanel.innerHTML.includes(arguments[0]) &&
-                        description && description.innerText.length > 0;    
-                ''',
-                job_id,
-                Selectors.detailsPanel,
-                Selectors.description)
+        try:
+            while elapsed < timeout:
+                loaded = driver.execute_script(
+                    '''
+                        const detailsPanel = document.querySelector(arguments[1]);
+                        const description = document.querySelector(arguments[2]);
+                        return detailsPanel && detailsPanel.innerHTML.includes(arguments[0]) &&
+                            description && description.innerText.length > 0;
+                    ''',
+                    job_id,
+                    Selectors.detailsPanel,
+                    Selectors.description)
 
-            if loaded:
-                return {'success': True}
+                if loaded:
+                    return {'success': True}
 
-            sleep(sleep_time)
-            elapsed += sleep_time
+                sleep(sleep_time)
+                elapsed += sleep_time
+        finally:
+            pass
 
         return {'success': False, 'error': 'Timeout on loading job details'}
 
     @staticmethod
-    def __paginate(driver: webdriver, tag, pagination_size=25, timeout=5) -> object:
+    def __paginate(driver: webdriver, current_url: str, tag: str, offset: int, timeout=5) -> object:
         try:
-            offset = int(get_query_params(driver.current_url)['start'])
-        except:
-            offset = 0
+            url = override_query_params(current_url, {'start': offset})
+            info(tag, f'Opening {url}')
+            driver.get(url)
 
-        offset += pagination_size
-        url = override_query_params(driver.current_url, {'start': offset})
-        info(tag, f'Next offset: {offset}')
-        info(tag, f'Opening {url}')
-        driver.get(url)
+            elapsed = 0
+            sleep_time = 0.05  # 50 ms
 
-        elapsed = 0
-        sleep_time = 0.05  # 50 ms
+            info(tag, f'Waiting for new jobs to load')
+            # Wait for new jobs to load
+            while elapsed < timeout:
+                loaded = driver.execute_script(
+                    '''
+                        return document.querySelectorAll(arguments[0]).length > 0;                
+                    ''',
+                    Selectors.jobs)
 
-        info(tag, f'Waiting for new jobs to load')
-        # Wait for new jobs to load
-        while elapsed < timeout:
-            loaded = driver.execute_script(
-                '''
-                    return document.querySelectorAll(arguments[0]).length > 0;                
-                ''',
-                Selectors.jobs)
+                if loaded:
+                    return {'success': True}
 
-            if loaded:
-                return {'success': True}
-
-            sleep(sleep_time)
-            elapsed += sleep_time
+                sleep(sleep_time)
+                elapsed += sleep_time
+        finally:
+            pass
 
         return {'success': False, 'error': 'Timeout on pagination'}
 
@@ -162,7 +190,16 @@ class AuthenticatedStrategy(Strategy):
         except:
             debug(tag, 'Failed to close chat panel')
 
-    def run(self, driver: webdriver, search_url: str, query: Query, location: str, apply_link: bool) -> None:
+    def run(
+        self,
+        driver: webdriver,
+        search_url: str,
+        query: Query,
+        location: str,
+        apply_link: bool,
+        page_load_timeout: int,
+        apply_page_load_timeout: int
+    ) -> None:
         """
         Run strategy
         :param driver: webdriver
@@ -170,12 +207,16 @@ class AuthenticatedStrategy(Strategy):
         :param query: Query
         :param location: str
         :param apply_link: bool
+        :param page_load_timeout: int
+        :param apply_page_load_timeout: int
         :return: None
         """
 
         tag = f'[{query.query}][{location}]'
         processed = 0
-        pagination_index = 1
+        skipped = 0
+        pagination_index = 0
+        pagination_size = 25
 
         # Open main page first to verify/set the session
         debug(tag, f'Opening {HOME_URL}')
@@ -239,17 +280,17 @@ class AuthenticatedStrategy(Strategy):
             # Jobs loop
             while job_index < job_tot and processed < query.options.limit:
                 sleep(self.scraper.slow_mo)
-                tag = f'[{query.query}][{location}][{processed + 1}]'
-
-                # Extract job main fields
-                debug(tag, 'Evaluating selectors', [
-                    Selectors.jobs,
-                    Selectors.link,
-                    Selectors.company,
-                    Selectors.place,
-                    Selectors.date])
+                tag = f'[{query.query}][{location}][{pagination_index * pagination_size + job_index + 1}]'
 
                 try:
+                    # Extract job main fields
+                    debug(tag, 'Evaluating selectors', [
+                        Selectors.jobs,
+                        Selectors.link,
+                        Selectors.company,
+                        Selectors.place,
+                        Selectors.date])
+
                     job_id, job_link, job_title, job_company, job_company_link, \
                     job_company_img_link, job_place, job_date = \
                         driver.execute_script(
@@ -321,8 +362,10 @@ class AuthenticatedStrategy(Strategy):
                     load_result = AuthenticatedStrategy.__load_job_details(driver, job_id)
 
                     if not load_result['success']:
-                        error(tag, load_result['error'])
+                        error(tag, load_result['error'], exc_info=False)
+                        info(tag, 'Skipped')
                         job_index += 1
+                        skipped += 1
                         continue
 
                     # Extract
@@ -353,77 +396,113 @@ class AuthenticatedStrategy(Strategy):
                     job_apply_link = ''
 
                     if apply_link:
-                        debug(tag, 'Evaluating selectors', [Selectors.applyBtn])
+                        try:
+                            debug(tag, 'Evaluating selectors', [Selectors.applyBtn])
 
-                        if driver.execute_script(
-                            r'''
-                                const applyBtn = document.querySelector(arguments[0]);
-                                
-                                if (applyBtn) {
-                                    applyBtn.click();
-                                    window.stop();
-                                    return true;
-                                }
-                                
-                                return false;                            
-                            ''',
-                            Selectors.applyBtn
-                        ) and len(driver.window_handles) > 1:
-                            debug(tag, 'Try extracting apply link')
-                            driver.switch_to.window(driver.window_handles[-1])  # Switch to apply page
-                            driver.execute_script('window.stop();')  # Stop page loading, we just want the url
-                            job_apply_link = driver.current_url
-                            driver.close()  # Close apply page
-                            driver.switch_to.window(driver.window_handles[0])  # Switch back to main page
+                            if driver.execute_script(
+                                r'''
+                                    const applyBtn = document.querySelector(arguments[0]);
+
+                                    if (applyBtn) {
+                                        applyBtn.click();
+                                        return true;
+                                    }
+
+                                    return false;
+                                ''',
+                                Selectors.applyBtn
+                            ) and len(driver.window_handles) > 1:
+                                debug(tag, 'Try extracting apply link')
+
+                                try:
+                                    # Trick to avoid wasting time loading apply page.
+                                    # It seems `driver.current_url` blocks until page is loaded, so we force a timeout
+                                    # and set `job_apply_link` in the except block. This doesn't work always: sometimes
+                                    # we get `about:blank` instead of the correct url.
+                                    driver.switch_to.window(driver.window_handles[-1])  # Switch to apply page
+                                    driver.set_page_load_timeout(apply_page_load_timeout)
+                                    job_apply_link = driver.current_url
+                                except:
+                                    job_apply_link = driver.current_url if driver.current_url != 'about:blank' else ''
+                        except BaseException as e:
+                            warn(tag, 'Failed to extract apply link', e)
+                        finally:
+                            driver.set_page_load_timeout(page_load_timeout)
+
+                            if len(driver.window_handles) > 1:
+                                driver.switch_to.window(driver.window_handles[-1])
+                                driver.close()
+                                driver.switch_to.window(driver.window_handles[0])
+
+                    data = EventData(
+                        query=query.query,
+                        location=location,
+                        job_id=job_id,
+                        job_index=job_index,
+                        title=job_title,
+                        company=job_company,
+                        company_link=job_company_link,
+                        company_img_link=job_company_img_link,
+                        place=job_place,
+                        date=job_date,
+                        link=job_link,
+                        apply_link=job_apply_link,
+                        description=job_description,
+                        description_html=job_description_html,
+                        insights=job_insights)
+
+                    info(tag, 'Processed')
+
+                    job_index += 1
+                    processed += 1
+
+                    self.scraper.emit(Events.DATA, data)
+
+                    # Try fetching more jobs
+                    if processed < query.options.limit and job_index == job_tot < pagination_size:
+                        load_jobs_result = AuthenticatedStrategy.__load_jobs(driver, job_tot)
+
+                        if load_jobs_result['success']:
+                            job_tot = load_jobs_result['count']
+
+                    if job_index == job_tot:
+                        break
 
                 except BaseException as e:
-                    # Verify session on error
-                    if not AuthenticatedStrategy.__is_authenticated_session(driver):
-                        warn(tag, 'Session is no longer valid, this may cause the scraper to fail')
-                        self.scraper.emit(Events.INVALID_SESSION)
+                    try:
+                        # Verify session on error
+                        if not AuthenticatedStrategy.__is_authenticated_session(driver):
+                            warn(tag, 'Session is no longer valid, this may cause the scraper to fail')
+                            self.scraper.emit(Events.INVALID_SESSION)
+
+                        error(tag, e, traceback.format_exc())
+                        self.scraper.emit(Events.ERROR, str(e) + '\n' + traceback.format_exc())
+                    finally:
+                        info(tag, 'Skipped')
+                        job_index += 1
+                        skipped += 1
 
-                    error(tag, e, traceback.format_exc())
-                    self.scraper.emit(Events.ERROR, str(e) + '\n' + traceback.format_exc())
-                    job_index += 1
                     continue
 
-                data = EventData(
-                    query=query.query,
-                    location=location,
-                    job_id=job_id,
-                    job_index=job_index,
-                    title=job_title,
-                    company=job_company,
-                    company_link=job_company_link,
-                    company_img_link=job_company_img_link,
-                    place=job_place,
-                    date=job_date,
-                    link=job_link,
-                    apply_link=job_apply_link,
-                    description=job_description,
-                    description_html=job_description_html,
-                    insights=job_insights)
+            tag = f'[{query.query}][{location}]'
 
-                info(tag, 'Processed')
+            info(tag, 'No more jobs to process in this page')
 
-                job_index += 1
-                processed += 1
-
-                self.scraper.emit(Events.DATA, data)
-
-                # Try fetching more jobs
-                if processed < query.options.limit and job_index == job_tot:
-                    job_tot = driver.execute_script('return document.querySelectorAll(arguments[0]).length;',
-                                                    Selectors.jobs)
+            # Print results so far
+            info(tag, 'Processed:', processed)
+            info(tag, 'Skipped:', skipped)
+            info(tag, 'Missed:', (pagination_index + 1) * pagination_size - processed - skipped)
 
             # Check if we reached the limit of jobs to process
             if processed == query.options.limit:
+                info(tag, 'Query limit reached!')
                 break
 
             # Try to paginate
             pagination_index += 1
-            info(tag, f'Pagination requested ({pagination_index})')
-            paginate_result = AuthenticatedStrategy.__paginate(driver, tag)
+            info(tag, f'Pagination requested [{pagination_index}]')
+            offset = pagination_index * pagination_size
+            paginate_result = AuthenticatedStrategy.__paginate(driver, search_url, tag, offset)
 
             if not paginate_result['success']:
                 info(tag, "Couldn't find more jobs for the running query")
